'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stringToNTriples = stringToNTriples;
exports.readNTriplesFile = readNTriplesFile;
exports.writeNTriplesFile = writeNTriplesFile;
exports.findObjectByPredicate = findObjectByPredicate;
exports.findLocalizedObjectByPredicate = findLocalizedObjectByPredicate;
exports.findObjectsByPredicate = findObjectsByPredicate;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _lines = require('underscore.string/lines');

var _lines2 = _interopRequireDefault(_lines);

var _include = require('underscore.string/include');

var _include2 = _interopRequireDefault(_include);

var _toBoolean = require('underscore.string/toBoolean');

var _toBoolean2 = _interopRequireDefault(_toBoolean);

var _toNumber = require('underscore.string/toNumber');

var _toNumber2 = _interopRequireDefault(_toNumber);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _n = require('n3');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isTriple = function isTriple(str) {
  return (0, _include2.default)(str, '<');
};

var omitGraph = function omitGraph(t) {
  return _lodash2.default.pick(t, ['subject', 'predicate', 'object']);
};
var withoutGraph = function withoutGraph(triples) {
  return _lodash2.default.map(triples, omitGraph);
};

var normalizeValue = function normalizeValue(value, defaultValue) {
  if (_lodash2.default.isNil(value)) {
    return defaultValue;
  }
  if (_n.Util.isLiteral(value)) {
    var litType = _n.Util.getLiteralType(value);
    var litValue = _n.Util.getLiteralValue(value);
    switch (litType) {
      case 'http://www.w3.org/2001/XMLSchema#string':
        return litValue;
      case 'http://www.w3.org/2001/XMLSchema#anyURI':
        return litValue;
      case 'http://www.w3.org/2001/XMLSchema#integer':
        return (0, _toNumber2.default)(litValue);
      case 'http://www.w3.org/2001/XMLSchema#nonPositiveInteger':
        return (0, _toNumber2.default)(litValue);
      case 'http://www.w3.org/2001/XMLSchema#negativeInteger':
        return (0, _toNumber2.default)(litValue);
      case 'http://www.w3.org/2001/XMLSchema#nonNegativeInteger':
        return (0, _toNumber2.default)(litValue);
      case 'http://www.w3.org/2001/XMLSchema#positiveInteger':
        return (0, _toNumber2.default)(litValue);
      case 'http://www.w3.org/2001/XMLSchema#float':
        return (0, _toNumber2.default)(litValue, 6);
      case 'http://www.w3.org/2001/XMLSchema#boolean':
        return (0, _toBoolean2.default)(litValue);
      case 'http://www.w3.org/2001/XMLSchema#dateTime':
        return (0, _moment2.default)(litValue, _moment2.default.ISO_8601);
      case 'http://www.w3.org/2001/XMLSchema#date':
        return (0, _moment2.default)(litValue, _moment2.default.ISO_8601);
      case 'http://www.w3.org/2001/XMLSchema#time':
        return litValue;
      default:
        return litValue;
    }
  }
  if (_n.Util.isIRI(value)) {
    return value;
  }
  return value;
};

var normalizeValues = function normalizeValues(values, defaultValue) {
  return _lodash2.default.map(values, function (v) {
    return normalizeValue(v, defaultValue);
  });
};

var asSemanticValue = function asSemanticValue(value) {
  var literalValue = normalizeValue(value, null);
  var literalType = _n.Util.getLiteralType(value);
  var literalLanguage = _n.Util.getLiteralLanguage(value);
  return { value: value, literalValue: literalValue, literalType: literalType, literalLanguage: literalLanguage };
};

var asSemanticValues = function asSemanticValues(values) {
  return _lodash2.default.map(values, asSemanticValue);
};

/**
 * Convert a string to an array of triples
 * @param {string} content - the n-triples as content
 * @returns {array} array of triples
 * @example
 * stringToNTriples(content);
 */
function stringToNTriples(content) {
  var n3parser = new _n.Parser();
  var n3parse = function n3parse(str) {
    return _lodash2.default.head(n3parser.parse(str));
  };

  var n3lines = _lodash2.default.filter((0, _lines2.default)(content), isTriple);
  var triples = _lodash2.default.map(n3lines, n3parse);
  return withoutGraph(triples);
}

/**
 * Reads a n-triples file and converts it to an array of triples
 * @param {string} filename - the n-triples filename
 * @param {callback} function - callback
 * @example
 * // [
 * { graph: "",
 * object: '"Flower corp"@en',
 * predicate: 'http://purl.org/dc/elements/1.1/publisher',
 * subject: 'http://www.site.org/version/123/'
 * },
 * ]
 * readNTriplesFile('flowers.nt', (err, triples) => {
 *  console.log(triples);
 * });
 */
function readNTriplesFile(filename, callback) {
  _fs2.default.readFile(filename, 'utf8', function (err, data) {
    if (err) {
      callback(err);
    } else {
      callback(null, stringToNTriples(data));
    }
  });
}

/**
 * Saves an array of triples in a n-triples file
 * @param {string} filename - the n-triples filename
 * @param {array} triples - an array of triples objects
 * @param {callback} function - callback
 * @example
 * // {count: 1}
 *  const triples = [
 * { graph: "",
 * object: '"Flower corp"@en',
 * predicate: 'http://purl.org/dc/elements/1.1/publisher',
 * subject: 'http://www.site.org/version/123/'
 * },
 * ]
 * writeNTriplesFile('flowers.nt', triples, (err, triples) => {
 *  console.log(triples);
 * });
 */
function writeNTriplesFile(filename, triples, callback) {
  var fstream = _fs2.default.createWriteStream(filename);
  var n3writer = new _n.Writer(fstream, { format: 'N-Triples' });
  n3writer.addTriples(withoutGraph(triples));
  n3writer.end(function (error) {
    var count = _lodash2.default.size(triples);
    var e = _lodash2.default.isNil(error) ? null : { error: error, filename: filename };
    callback(e, { count: count });
  });
}

/**
 * Finds the first object value based on the predicate
 * @param {array} triples - an array of triples objects (subject is ignored)
 * @param {string} predicate - the uri representing the predicate
 * @param {object} defaultValue - the object/string to return if null
 * @example
 * // returns Amadeus
 * findObjectByPredicate(triples, 'http://purl.org/dc/elements/1.1/creator')
 * @return {string} the string, integer, float, boolean, moment representing the literal value
 */
function findObjectByPredicate(triples, predicate) {
  var defaultValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  var value = _lodash2.default.get(_lodash2.default.find(triples, { predicate: predicate }), 'object');
  return normalizeValue(value, defaultValue);
}

/**
 * Finds the first object value based on the predicate and the language
 * @param {array} triples - an array of triples objects (subject is ignored)
 * @param {string} predicate - the uri representing the predicate
 * @param {string} language - the requested language
 * @param {array} altLangs - an array of alternative languages (max 2)
 * @param {object} defaultValue - the object/string to return if null
 * @example
 * // returns Amadeus
 * findLocalizedObjectByPredicate(triples, 'http://purl.org/dc/elements/1.1/creator', 'fr', ['en'])
 * @return {string} the string, integer, float, boolean, moment representing the literal value
 */
function findLocalizedObjectByPredicate(triples, predicate, language, altLangs) {
  var defaultValue = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;

  var values = _lodash2.default.map(_lodash2.default.filter(triples, { predicate: predicate }), 'object');
  if (_lodash2.default.isEmpty(values)) {
    return defaultValue;
  }

  var semanticValues = asSemanticValues(values);
  var altLang = _lodash2.default.nth(altLangs);
  var altLang2 = _lodash2.default.nth(altLangs, 1);
  var value = _lodash2.default.find(semanticValues, { literalLanguage: language });
  if (_lodash2.default.isNil(altLang) || !_lodash2.default.isNil(value)) {
    return _lodash2.default.get(value, 'literalValue', defaultValue);
  }
  var altValue = _lodash2.default.find(semanticValues, { literalLanguage: altLang });
  if (_lodash2.default.isNil(altLang2) || !_lodash2.default.isNil(altValue)) {
    return _lodash2.default.get(altValue, 'literalValue', defaultValue);
  }
  var altValue2 = _lodash2.default.find(semanticValues, { literalLanguage: altLang2 });
  return _lodash2.default.get(altValue2, 'literalValue', defaultValue);
}

/**
 * Finds all object values based on the predicate
 * @param {array} triples - an array of triples objects (subject is ignored)
 * @param {string} predicate - the uri representing the predicate
 * @example
 * // returns [Amadeus, Bach]
 * findObjectsByPredicate(triples, 'http://purl.org/dc/elements/1.1/creator')
 * @return {array} of string, integer, float, boolean, moment representing the literal values
 */
function findObjectsByPredicate(triples, predicate) {
  var values = _lodash2.default.map(_lodash2.default.filter(triples, { predicate: predicate }), 'object');
  return normalizeValues(values);
}